@@ -25,7 +25,7 @@
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
-
+from __future__ import with_statement
 import datetime
 import logging
 import sys
@@ -36,6 +36,9 @@ import socket
 import urllib
 import urllib2
 import uuid
+import ConfigParser
+import os
+import decorator
 
 from errormator_client.ext_json import json
 from errormator_client.utils import asbool, aslist
@@ -55,10 +58,10 @@ log = logging.getLogger(__name__)
 
 
 class Client(object):
-    __version__ = '0.4.1'
+    __version__ = '0.5'
     __protocol_version__ = '0.3'
 
-    def __init__(self, config):
+    def __init__(self, config=None, register_timing=True):
         """
         at minimum client expects following keys to be present::
         
@@ -66,7 +69,7 @@ class Client(object):
             errormator.server_url = https://api.errormator.com
             errormator.api_key = YOUR_API_KEY
 
-        additional config keys you can set in config object::
+        additional config keys you can set::
 
             errormator.server_name - identifier for Instance/Server Name your application is running on (default: auto determined fqdn of server)
             errormator.timeout - connection timeout when communicating with API
@@ -75,7 +78,7 @@ class Client(object):
             errormator.logging - enable hooking to application loggers
             errormator.logging.level - minimum log level for log capture
             errormator.logging_on_error - send logs only from erroneous/slow requests (default false) 
-            errormator.slow_request_time - (float/int) time in seconds after request is considered being slow (default 3)
+            errormator.slow_request_time - (float/int) time in seconds after request is considered being slow (default 1)
             errormator.report_404 - enables 404 error logging (default False)
             errormator.report_errors - enables 500 error logging (default True)
             errormator.buffer_flush_interval - how often send data to mothership Errormator (default 5)
@@ -99,7 +102,7 @@ class Client(object):
         self.config['reraise_exceptions'] = asbool(
                 config.get('errormator.reraise_exceptions', True))
         self.config['slow_requests'] = asbool(config.get('errormator.slow_requests', True))
-        self.config['slow_request_time'] = float(config.get('errormator.slow_request.time', 3))
+        self.config['slow_request_time'] = float(config.get('errormator.slow_request.time', 1))
         if self.config['slow_request_time'] < 0.01:
             self.config['slow_request_time'] = 0.01
         self.config['slow_request_time'] = datetime.timedelta(seconds=self.config['slow_request_time'])
@@ -175,64 +178,50 @@ class Client(object):
         self.uuid = uuid.uuid4()
         self.last_submit = datetime.datetime.now()
 
-    def submit_report_data(self):
-        def send():
-            with self.report_queue_lock:
-                to_send_items = self.report_queue[:250]
-                self.report_queue = self.report_queue[250:]
-            if to_send_items:
-                try:
-                    self.remote_call(to_send_items, self.endpoints['reports'])
-                except KeyboardInterrupt as e:
-                    raise KeyboardInterrupt()
-                except Exception as e:
-                    log.warning('REPORTS: connection issue: %s' % e)
-                    return False
-            return True
-        return send()
-        # FIXME: reintroduce threads
+    def submit_data(self):
+        results = {'reports':False,
+                   'logs':False,
+                   'slow_reports':False
+                   }
+        with self.report_queue_lock:
+            reports = self.report_queue[:250]
+            self.report_queue = self.report_queue[250:]
+        with self.slow_report_queue_lock:
+            slow_reports = self.slow_report_queue[:250]
+            self.slow_report_queue = self.slow_report_queue[250:]
+        with self.log_queue_lock:
+            logs = self.log_queue[:2000]
+            self.log_queue = self.log_queue[2000:]
+        results['reports'] = self.api_create_submit(reports, 'reports')
+        results['slow_reports'] = self.api_create_submit(slow_reports,
+                                                         'slow_reports')
+        results['logs'] = self.api_create_submit(logs, 'logs')
+        return results
 
-    def submit_other_data(self):
-        def send():
-            with self.slow_report_queue_lock:
-                slow_to_send_items = self.slow_report_queue[:250]
-                self.slow_report_queue = self.slow_report_queue[250:]
-            with self.log_queue_lock:
-                logs_to_send_items = self.log_queue[:2000]
-                self.log_queue = self.log_queue[2000:]
-
-            if slow_to_send_items:
-                try:
-                    self.remote_call(slow_to_send_items,
-                                 self.endpoints['slow_reports'])
-                except KeyboardInterrupt as e:
-                    raise KeyboardInterrupt()
-                except Exception as e:
-                    log.warning('SLOW REPORTS: connection issue: %s' % e)
-            if logs_to_send_items:
-                try:
-                    self.remote_call(logs_to_send_items,
-                                 self.endpoints['logs'])
-                except KeyboardInterrupt as e:
-                    raise KeyboardInterrupt()
-                except Exception as e:
-                    log.warning('LOGS: connection issue: %s' % e)
-            return True
-        return send()
-        # FIXME: reintroduce threads
+    def api_create_submit(self, to_send_items, endpoint):
+        if to_send_items:
+            try:
+                self.remote_call(to_send_items, self.endpoints[endpoint])
+            except KeyboardInterrupt as e:
+                raise KeyboardInterrupt()
+            except Exception as e:
+                log.warning('%s: connection issue: %s' % (endpoint, e))
+                return False
+        return True
 
     def check_if_deliver(self, force_send=False):
         delta = datetime.datetime.now() - self.last_submit
         if delta > self.config['buffer_flush_interval'] or force_send:
-            submit_report_data_t = threading.Thread(target=self.submit_report_data)
-            submit_report_data_t.start()
-            submit_other_data_t = threading.Thread(target=self.submit_other_data)
-            submit_other_data_t.start()
+            submit_data_t = threading.Thread(target=self.submit_data)
+            submit_data_t.start()
             self.last_submit = datetime.datetime.now()
             return True
         return False
 
     def remote_call(self, data, endpoint):
+        if not self.config['api_key']:
+            log.warning('no api key set - dropping payload')
+            return False          
         GET_vars = urllib.urlencode({'protocol_version': self.__protocol_version__})
         server_url = '%s%s?%s' % (self.config['server_url'], endpoint, GET_vars,)
         headers = {'content-type': 'application/json',
@@ -424,15 +413,62 @@ class Client(object):
         report_data.update(errormator_info)
         return report_data, errormator_info
 
+def get_config(config=None, path_to_config=None, section_name='errormator'):
+    if not config and not path_to_config:
+        path_to_config = os.environ.get('ERRORMATOR_INI')
+    if path_to_config:
+        config = {}
+        if not os.path.exists(path_to_config):
+            log.warning("Couldn't locate %s " % path_to_config)
+            return config            
+        with open(path_to_config) as f:
+            parser = ConfigParser.SafeConfigParser()
+            parser.readfp(f)
+            try:
+                config = dict(parser.items(section_name))
+            except ConfigParser.NoSectionError as e:
+                log.warning('No section name called %s in file' % section_name)
+            return config                    
+    return config or {}
 
-def make_errormator_middleware(app, global_config, **kw):
-    config = global_config.copy()
+def decorate(ini_file=None, register_timing=True):
+    def make_errormator_middleware(app, global_config={}, **kw):
+        config = global_config.copy()
+        config.update(kw)
+        #this shuts down all errormator functionalities
+        if not asbool(config.get('errormator', True)):
+            return app
+        log.critical('ERRORMATOR MIDDLEWARE')
+        ini_path = os.environ.get('ERRORMATOR_INI',
+                                  config.get('errormator.config_path',
+                                             ini_file))
+        config = get_config(config=config, path_to_config=ini_path)
+        client = Client(config)
+        from errormator_client.wsgi import ErrormatorWSGIWrapper
+        app = ErrormatorWSGIWrapper(app, client)
+        return app    
+    def foo(f):
+        return decorator.decorator(make_errormator_middleware, f)
+    return foo
+    
+#TODO: refactor this to share the code
+def make_errormator_middleware(app, global_config={}, **kw):
+    if global_config:
+        config = global_config.copy()
+    else:
+        config = {}
     config.update(kw)
+    ini_path = os.environ.get('ERRORMATOR_INI',
+                              config.get('errormator.config_path'))
+    config = get_config(config=config, path_to_config=ini_path)    
     #this shuts down all errormator functionalities
-    if not asbool(config.get('errormator', True)):
+    if not asbool(config.get('errormator', False)):
+        log.warning('''ERRORMATOR_INI config variable is missing from 
+                    environment or errormator.config_path 
+                    not passed in app global config, 
+                    or errormator disabled in config''')
         return app
-
-    client = Client(config=config)
+    client = Client(config)
     from errormator_client.wsgi import ErrormatorWSGIWrapper
     app = ErrormatorWSGIWrapper(app, client)
     return app
