@@ -1,9 +1,12 @@
 from __future__ import absolute_import
 from flask import request
 from flask.signals import got_request_exception
-from errormator_client import make_errormator_middleware
+from errormator_client.client import make_errormator_middleware, get_config
 from errormator_client.ext.general import gather_data
 
+import logging
+
+log = logging.getLogger(__name__)
 
 def log_exception(sender, exception, **extra):
     errormator_client = request.environ['errormator.client']
@@ -11,7 +14,7 @@ def log_exception(sender, exception, **extra):
                 gather_logs=False)
 
 
-def add_errormator(app, config):
+def add_errormator(app, config=None):
     """
         Adds Errormator to Flask,
         
@@ -20,7 +23,7 @@ def add_errormator(app, config):
     if not config:
         config = app.config.get('ERRORMATOR')
     if not config:
-        raise Exception("Couldn't find Errormator config")
+        config = get_config()
     app.wsgi_app = make_errormator_middleware(app.wsgi_app, config)
     got_request_exception.connect(log_exception, app)
     return app
