@@ -2,11 +2,22 @@ import unittest
 import datetime
 import logging
 import socket
-from errormator_client import client
+import pkg_resources
+from errormator_client import client, make_errormator_middleware
 from errormator_client.exceptions import get_current_traceback
 from errormator_client.logger import register_logging
+from errormator_client.wsgi import ErrormatorWSGIWrapper
 
 
+fname = pkg_resources.resource_filename('errormator_client',
+                                        'templates/default_template.ini')
+timing_conf = client.get_config(path_to_config=fname)
+for k,v in timing_conf.iteritems(): 
+    if 'errormator.timing' in k:
+        timing_conf[k] = 0.000001
+
+client.Client(config=timing_conf)
+from errormator_client.timing import local_timing
 
 def example_filter_callable(structure, section=None):
     return 'filtered-data'
@@ -39,6 +50,10 @@ TEST_ENVIRON = {
 'REMOTE_USER':'foo'
 }
 
+REQ_START_TIME = datetime.datetime(2012, 9, 26, 18, 17, 54, 461254)
+REQ_END_TIME = datetime.datetime(2012, 9, 26, 18, 18, 4, 461259)
+SERVER_NAME = socket.getfqdn() # different on every machine
+
 PARSED_REPORT_404 = {
                      'report_details': [{'username': '',
                         'url': 'http://localhost:6543/test/error?aaa=1&bbb=2',
@@ -51,7 +66,7 @@ PARSED_REPORT_404 = {
                                     'GET': {u'aaa': [u'1'], u'bbb': [u'2']}},
                         'user_agent': u'', 'message': u''}],
                      'error_type': '404 Not Found',
-                     'server': socket.getfqdn(), # different on every machine
+                     'server': SERVER_NAME,
                      'priority': 5,
                      'client': 'Python',
                      'http_status': 404}
@@ -77,11 +92,32 @@ PARSED_REPORT_500 = {'traceback': u'Traceback (most recent call last):', #this w
                                          'user_agent': u'Mozilla/5.0 (X11; Linux x86_64; rv:10.0.1) Gecko/20100101 Firefox/10.0.1',
                                          'message': u''}],
                      'error_type': u'Exception: Test Exception',
-                     'server': 'ergo-webreactor',
+                     'server': SERVER_NAME,
                      'priority': 5,
                      'client': 'Python',
                      'http_status': 500}
 
+PARSED_SLOW_REPORT = {
+                      'report_details': [{'username': '',
+                                          'url': 'http://localhost:6543/test/error?aaa=1&bbb=2',
+                                          'ip': '127.0.0.1',
+                                          'start_time': REQ_START_TIME,
+                                          'slow_calls': [],
+                                          'request': {'COOKIES': {u'country': u'US',
+                                                                  u'sessionId': u'***',
+                                                                  u'test_group_id': u'5',
+                                                                  u'http_referer': u'http://localhost:5000/'},
+                                                      'POST': {},
+                                                      'GET': {u'aaa': [u'1'], u'bbb': [u'2']}},
+                                          'user_agent': u'',
+                                          'message': u'',
+                                          'end_time': REQ_END_TIME}],
+                      'error_type': 'Unknown',
+                      'server': SERVER_NAME,
+                      'priority': 5,
+                      'client': 'Python',
+                      'http_status': 200}
+
 class TestClientConfig(unittest.TestCase):
 
     def setUpClient(self, config={}):
@@ -168,7 +204,7 @@ class TestClientConfig(unittest.TestCase):
     def test_default_slow_request_time(self):
         self.setUpClient()
         self.assertEqual(self.client.config['slow_request_time'],
-                         datetime.timedelta(seconds=3))
+                         datetime.timedelta(seconds=1))
 
     def test_custom_slow_request_time(self):
         config = {'errormator.slow_request.time':"2"}
@@ -307,7 +343,7 @@ class TestClientConfig(unittest.TestCase):
 
     def test_default_logging_handler_level(self):
         self.setUpClient()
-        self.assertEqual(self.client.log_handler.level, logging.NOTSET)
+        self.assertEqual(self.client.log_handler.level, logging.WARNING)
 
     def test_custom_logging_handler_level(self):
         config = {'errormator.logging.level':"CRITICAL"}
@@ -342,7 +378,7 @@ def generate_error():
 
 class TestClientSending(unittest.TestCase):
 
-    def setUpClient(self, config={}):
+    def setUpClient(self, config={'errormator.api_key':'blargh!'}):
         self.client = client.Client(config)
 
     def test_check_if_deliver_false(self):
@@ -353,6 +389,23 @@ class TestClientSending(unittest.TestCase):
         self.setUpClient()
         self.assertEqual(self.client.check_if_deliver(force_send=True), True)
 
+    def test_send_error_failure_queue(self):
+        self.setUpClient()
+        self.client.py_report(TEST_ENVIRON, http_status=404)
+        result = self.client.submit_data()
+        self.assertEqual(self.client.report_queue, [])
+
+    def test_send_error_failure(self):
+        self.setUpClient()
+        self.client.py_report(TEST_ENVIRON, http_status=404)
+        result = self.client.submit_data()
+        self.assertEqual(result['reports'], False)
+        
+    def test_send_error_io(self):
+        self.setUpClient()
+        self.client.py_report(TEST_ENVIRON, http_status=404)
+        result = self.client.submit_data()
+        self.assertEqual(result['reports'], False)
 
 class TestErrorParsing(unittest.TestCase):
     def setUpClient(self, config={}):
@@ -404,15 +457,305 @@ class TestLogs(unittest.TestCase):
         self.client.log_queue[0]['server'] = fake_log['server']
         self.assertEqual(self.client.log_queue[0], fake_log)
 
-class TestSending(unittest.TestCase):
+class TestSlowReportParsing(unittest.TestCase):
     def setUpClient(self, config={}):
         self.client = client.Client(config)
 
-    def test_send_error_failure(self):
+    def test_py_report_slow(self):
         self.setUpClient()
-        self.client.py_report(TEST_ENVIRON, http_status=404)
-        self.client.submit_report_data()
-        self.assertEqual(self.client.report_queue, [])
+        self.maxDiff = None
+        self.client.py_slow_report(TEST_ENVIRON, start_time=REQ_START_TIME,
+                                   end_time=REQ_END_TIME)
+        self.assertEqual(self.client.slow_report_queue[0], PARSED_SLOW_REPORT)
+
+class TestMakeMiddleware(unittest.TestCase):
+    
+    def test_make_middleware(self):
+        def app(environ, start_response):
+            start_response('200 OK', [('content-type', 'text/html')])
+            return ['Hello world!']
+        app = make_errormator_middleware(app, {'errormator':True})
+        self.assertTrue(isinstance(app, ErrormatorWSGIWrapper))
+
+    def test_make_middleware_disabled(self):
+        def app(environ, start_response):
+            start_response('200 OK', [('content-type', 'text/html')])
+            return ['Hello world!']
+        app = make_errormator_middleware(app, {})        
+        self.assertFalse(isinstance(app, ErrormatorWSGIWrapper))
+
+
+class TestTimingHTTPLibs(unittest.TestCase):
+    
+    def setUpClient(self, config={}):
+        self.client = client.Client(config)
+        
+    def setUp(self):
+        self.setUpClient(timing_conf)
+    
+    def test_urllib_URLOpener_open(self):
+        import urllib
+        opener = urllib.URLopener()
+        f = opener.open("http://www.ubuntu.com/")
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_urllib_urlretrieve(self):
+        import urllib
+        urllib.urlretrieve("http://www.ubuntu.com/")
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_urllib2(self):
+        import urllib2
+        urllib2.urlopen("http://www.ubuntu.com/")
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_urllib3(self):
+        import urllib3
+        http = urllib3.PoolManager()
+        r = http.request('GET', "http://www.ubuntu.com/")
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+        
+    def test_requests(self):
+        import requests
+        r = requests.get("http://www.ubuntu.com/")
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_httplib(self):
+        import httplib
+        h2 = httplib.HTTPConnection("www.ubuntu.com")
+        h2.request("GET", "/")
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+class TestDBApi2Drivers(unittest.TestCase):
+    
+    def setUpClient(self, config={}):
+        self.client = client.Client(config)
+    
+    def setUp(self):
+        self.setUpClient(timing_conf)
+        self.stmt = '''SELECT 1+2+3 as result''' 
+    
+    def test_sqlite(self):
+        try:
+            import sqlite3
+        except ImportError:
+            return
+        conn = sqlite3.connect(':memory:')
+        c = conn.cursor()
+        c.execute(self.stmt)
+        c.fetchone()
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+    
+    def test_psycopg2(self):
+        try:
+            import psycopg2
+        except ImportError:
+            return
+        conn = psycopg2.connect("user=postgres host=127.0.0.1")
+        c = conn.cursor()
+        psycopg2.extensions.register_type(psycopg2.extensions.UNICODE, c)
+        c.execute(self.stmt)
+        c.fetchone()
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+        
+    def test_pg8000(self):
+        try:
+            import pg8000
+        except ImportError:
+            return
+        conn = pg8000.DBAPI.connect(host="localhost", user="test", password="test")
+        c = conn.cursor()
+        c.execute(self.stmt)
+        c.fetchone()
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+        
+    def test_postgresql(self):
+        try:
+            import postgresql
+        except ImportError:
+            return
+        conn = postgresql.open('pq://test:test@localhost')
+        c = conn.cursor()
+        c.execute(self.stmt)
+        c.fetchone()[0]
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_mysqldb(self):
+        try:
+            import MySQLdb
+        except ImportError:
+            return
+        conn = MySQLdb.connect(passwd="test", user="test")
+        c = conn.cursor()
+        c.execute(self.stmt)
+        c.fetchone()
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_oursql(self):
+        import oursql
+        conn = oursql.connect(passwd="test", user="test")
+        c = conn.cursor(oursql.DictCursor)
+        c.execute(self.stmt)
+        c.fetchone()
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_odbc(self):
+        try:
+            import pyodbc
+        except ImportError:
+            return
+        conn = pyodbc.connect('Driver={MySQL};Server=127.0.0.1;Port=3306;Database=information_schema;User=test; Password=test;Option=3;')
+        c = conn.cursor()
+        c.execute(self.stmt)
+        c.fetchone()
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_pymysql(self):
+        try:
+            import pymysql
+        except ImportError:
+            return
+        conn = pymysql.connect(host='127.0.0.1', user='test', passwd='test')
+        c = conn.cursor()
+        c.execute(self.stmt)
+        c.fetchone()
+        c.close()
+        conn.close()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+
+class TestMako(unittest.TestCase):
+   
+    def setUpClient(self, config={}):
+        self.client = client.Client(timing_conf)
+        
+    def setUp(self):
+        self.setUpClient(timing_conf)
+   
+    def test_render(self):
+        try:
+            import mako
+        except ImportError:
+            return
+        template = mako.template.Template('''
+        <%
+        import time
+        time.sleep(0.01)
+        %> 
+        xxxxx ${1+2} yyyyyy
+        
+        ''')
+        template.render()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+    def test_render_unicode(self):
+        try:
+            import mako
+        except ImportError:
+            return
+        template = mako.template.Template(u'''
+        <%
+        import time
+        time.sleep(0.01)
+        %> 
+        xxxxx ${1+2} yyyyyy
+        
+        ''')
+        template.render_unicode()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)        
+
+    def test_template_lookup(self):
+        try:
+            import mako
+        except ImportError:
+            return
+        from mako.lookup import TemplateLookup
+        lookup = TemplateLookup()
+        lookup.put_string("base.html", '''
+        <%
+        import time
+        time.sleep(0.01)
+        %> 
+            <html><body></body></html>
+        ''')
+        template = lookup.get_template("base.html")
+        template.render_unicode()
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)     
+
+class TestJinja2(unittest.TestCase):
+   
+    def setUpClient(self, config={}):
+        self.client = client.Client(timing_conf)
+   
+    def test_render(self):
+        try:
+            import jinja2
+        except ImportError:
+            return
+        import time
+        template = jinja2.Template('''
+        {{sleep(0.06)}}
+        xxxxx {{1+2}} yyyyyy
+        ''')
+        template.render(sleep=time.sleep)
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
+
+class TestDjangoTemplates(unittest.TestCase):
+   
+    def setUpClient(self, config={}):
+        self.client = client.Client(config)
+        
+    def setUp(self):
+        self.setUpClient(timing_conf)
+   
+    def test_render(self):
+        try:
+            from django import template
+        except ImportError:
+            return
+        from django.conf import settings
+        settings.configure(TEMPLATE_DIRS=("/whatever/templates",))
+        import time
+        ctx = template.Context()
+        ctx.update({'time':lambda :time.sleep(0.06)})
+        template = template.Template('''
+        xxxxx {{ time }} yyyyyy
+        ''')
+        template.render(ctx)
+        result = local_timing._errormator.get_slow_calls()
+        self.assertEqual(len(result), 1)
 
 if __name__ == '__main__':
     unittest.main()  # pragma: nocover
